bl_info = {
    "name" : "Partybots converter",
    "author" : "vuyghu2020",
    "version" : (0, 1),
    "blender" : (4, 1, 1),
    "location" : "View3d > Tool",
    "warning" : "",
    "wiki_url" : "",
    "category" : "Partybots map builder",
}

import bpy
import math





def get_property(obj, prop_name, default_value=None):
    return obj.get(prop_name, default_value)







class ADD_PLATFORM_DIALOG(bpy.types.Operator):
    '''e'''
    bl_label = "Add platform"
    bl_idname = "vuyghu2020.dialog"
    
    Location: bpy.props.FloatVectorProperty(name = "Location", subtype='XYZ', default=(0.0,0.0,0.5), precision=3)
    Rotation: bpy.props.IntVectorProperty(name = "Rotation", subtype='XYZ')
    Scale: bpy.props.FloatVectorProperty(name = "Scale", subtype='XYZ', default=(1.0,1.0,1.0), precision=3)
    
    type: bpy.props.EnumProperty(name = "Material", items=[
            ('grass', 'Grass', "The default"),
            ('snow', 'Snow', "Colder then ice (Will melt in lava)"),
            ('slime', 'Slime', "Sticky (Will make you slower and immune to teleportation)"),
            ('steel', 'Steel', "Steely and hot in lava"),
            ('wood', 'Wood', "Burns easier")
        ])
    conveyor: bpy.props.BoolProperty(name = "Conveyor")
    conveyor_velocity: bpy.props.FloatProperty(name = "Conveyor velocity", precision=3, min = 0.1, max = 10, default = 0.5)
    conveyor_angle: bpy.props.FloatProperty(name = "Conveyor angle", precision=3, min = -360, max = 360, default = -180)
    
    corner_cut: bpy.props.BoolProperty(name = "Corner cut")
    corner_cut_radius: bpy.props.FloatProperty(name = "Corner cut radius", precision=3, min = 1, max = 15, default = 5.8)
    
    corner_cut_shapes: bpy.props.EnumProperty(name = "Corner cut shape", items=[
            ('none', 'None', "Nouthing changes"),
            ('circle', 'Circle', ""),
            ('square', 'Square', ""),
            ('rectangle', 'Rectangle', ""),
            ('diamond', 'Diamond', ""),
            ('hexagon', 'Hexagon', ""),
            ('plus', 'Plus', ""),
            ('star', 'Star', ""),
            ('heart', 'Heart', ""),
            ('ring', 'Ring', "")
        ])
        
    hole: bpy.props.BoolProperty(name = "Hole")
    hole_radius: bpy.props.FloatProperty(name = "Hole radius", precision=3, min = 1, max = 15, default = 5.8)

    hole_shapes: bpy.props.EnumProperty(name = "Hole shape", items=[
            ('none', 'None', "Nouthing changes"),
            ('circle', 'Circle', ""),
            ('square', 'Square', ""),
            ('rectangle', 'Rectangle', ""),
            ('diamond', 'Diamond', ""),
            ('hexagon', 'Hexagon', ""),
            ('plus', 'Plus', ""),
            ('star', 'Star', ""),
            ('heart', 'Heart', ""),
            ('ring', 'Ring', "")
        ])

    
    def execute(self, context):
        bpy.ops.mesh.primitive_cube_add()
        obj = bpy.context.object
        obj.location = [Location * 2 for Location in self.Location]
        obj.rotation_euler = [grader * (math.pi / 180) for grader in self.Rotation]
        obj.scale = self.Scale
        obj["type"] = self.type
        obj["conveyor"] = self.conveyor
        obj["conveyor_velocity"] = round(self.conveyor_velocity, 3)
        obj["conveyor_angle"] = round(self.conveyor_angle, 3)
        obj["corner_cut"] = self.corner_cut
        obj["corner_cut_radius"] = round(self.corner_cut_radius, 3)
        obj["corner_cut_shapes"] = self.corner_cut_shapes
        obj["hole"] = self.hole
        obj["hole_radius"] = round(self.hole, 3)
        obj["hole_shapes"] = self.hole_shapes
        
        
        return {'FINISHED'}
    
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

"""
class CHANGE_PROPERTYS_SELECTED_BUTTON(bpy.types.Operator):
    bl_label = "Change propertys of selected"
    bl_idname = "vuyghu2020.change_propertys_selected_button"
    
    def execute(self, context):
        bpy.ops.vuyghu2020.change_propertys_dialog('INVOKE_DEFAULT')
        return {'FINISHED'}
"""



class ADD_PLATFORM_BUTTON(bpy.types.Operator):
    bl_label = "Add platform"
    bl_idname = "vuyghu2020.add_platform_button"
    
    def execute(self, context):
        bpy.ops.vuyghu2020.dialog('INVOKE_DEFAULT')
        return {'FINISHED'}


class COMPILE_SELECTED(bpy.types.Operator):
    bl_label = "Compile selected to clipbord"
    bl_idname = "vuyghu2020.compile_selected"
    
    def execute(self, context):
        
        objs = bpy.context.selected_objects
        e = '''{"RENDER_MODEL", ""}
 [info]
 color 1
 [trigger]
 time 0
 alert_type none
'''
        degre_angel = 1/((math.pi*2)/360) # This line is created whith the help of Observin check his Twitch https://www.twitch.tv/observin_
        for x in objs:
                
            e = e +f'''{{"O_{x.name}", ""}}
 [info]
 color 0
 [general]
 platform_size {x.scale[0]}
 platform_vertical_size {x.scale[2]}
 platform_materials {get_property(x, "type", "grass")}
 [?global.new_platform]
 x {round(x.location[0]/2,3)}
 y {round(x.location[2]/2,3)}
 z {round(x.location[1]/2,3)}
 spawn_in_place true
 x_rotation {round(x.rotation_euler[0] * degre_angel*-1)}
 y_rotation {round(x.rotation_euler[2] * degre_angel*-1)}
 z_rotation {round(x.rotation_euler[1] * degre_angel*-1)}
 [trigger]
 relative_time 0
 alert_type none
'''
            
            if get_property(x, "conveyor", False) or get_property(x, "corner_cut", False):
                
                e = e +f'''{{"C_{x.name}", ""}}
 [info]
 color 0
 [trigger]
 recipients notriggered
 relative_time 0
 alert_type none
 '''
                if get_property(x, "conveyor", False):
                    print("eeee")
                    e = e +f'''[?platform.conveyor]
 angle {get_property(x, "conveyor_angle", -180.0)}
 velocity {get_property(x, "conveyor_velocity", 0.5)}
'''
                if get_property(x, "corner_cut", False):
                    e = e +f''' [?platform.corner_cut]
 shapes {get_property(x, "corner_cut_shapes", "null")}
 radius {get_property(x, "corner_cut_radius", 5.8)}
'''
                if get_property(x, "hole", False):
                    e = e +f''' [?platform.hole]
 shapes {get_property(x, "hole_shapes", "null")}
 radius {get_property(x, "hole_radius", 4)}
'''

        print(e)
        bpy.context.window_manager.clipboard = e
        print("Eee")
        

        

        
        
        return {'FINISHED'}
    
    




class TestPanel(bpy.types.Panel):
    bl_label = "Partybots converter panel"
    bl_idname = "TestPanel"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Partybots converter"
    
    def draw(self, context):
        leyout = self.layout
        
        row = leyout.row()
        row.operator("vuyghu2020.add_platform_button")
        #row = leyout.row()
        #row.operator("vuyghu2020.change_propertys_selected_button")
        row = leyout.row()
        row.operator("vuyghu2020.compile_selected")
        row = leyout.row()
        row.label(text = "Made by vuyghu2020")
        
        
        
        
        

        

def register():

    
    
    
    bpy.utils.register_class(TestPanel)
    #bpy.utils.register_class(CHANGE_PROPERTYS_DIALOG)
    #bpy.utils.register_class(CHANGE_PROPERTYS_SELECTED_BUTTON)
    bpy.utils.register_class(ADD_PLATFORM_DIALOG)
    bpy.utils.register_class(ADD_PLATFORM_BUTTON)
    bpy.utils.register_class(COMPILE_SELECTED)
    
    

    
    
    
    
    
def unregister():
    

    
    bpy.utils.unregister_class(TestPanel)
    #bpy.utils.unregister_class(CHANGE_PROPERTYS_DIALOG)
    #bpy.utils.unregister_class(CHANGE_PROPERTYS_SELECTED_BUTTON)
    bpy.utils.unregister_class(ADD_PLATFORM_DIALOG)
    bpy.utils.unregister_class(ADD_PLATFORM_BUTTON)
    bpy.utils.unregister_class(COMPILE_SELECTED)
    

    
    
    
if __name__ == "__main__":
    register()
    
